!
!   Jose Paulo Bonatti - DMD/CPTEC - May 2011
!
MODULE Decomposition

  USE InputParameters, ONLY : r8, &
                              Mend1 => Mend1Out, Mend2 => Mend2Out, Mnwv2 => Mnwv2Out, &
                              Imax => ImaxOut, Jmax => JmaxOut, Kmax => KmaxOut, &
                              Imx => ImxOut, Jmaxhf => JmaxhfOut, ICaseRec

  USE Fourier, ONLY : fft991

  USE Legendre, ONLY : SymAsy, FourierToSpecCoef, &
                       GetConv, GetVort, la0, colrad


  IMPLICIT NONE

  PRIVATE

  PUBLIC :: DectoSpherHarm, UVtoDivgVort, TransSpherHarm
 

CONTAINS


SUBROUTINE DectoSpherHarm (f, qf, Ldim)

  IMPLICIT NONE

  INTEGER, INTENT (IN) :: Ldim

  REAL (KIND=r8), DIMENSION (Imax,Jmax,Ldim), INTENT (IN)  :: f
  REAL (KIND=r8), DIMENSION (Mnwv2,Ldim),     INTENT (OUT) :: qf

  INTEGER :: i, j, k, inc

  REAL (KIND=r8), DIMENSION (Imx,Ldim) :: fn, fs, gn, gs
  REAL (KIND=r8), DIMENSION (Imx*Ldim) :: fna, fsa, gna, gsa

  gn=0.0_r8
  gs=0.0_r8
  qf=0.0_r8
  inc=1

  DO j=1,Jmaxhf
     DO k=1,Ldim
        DO i=1,Imax
           fn(i,k)=f(i,j,k)
           fs(i,k)=f(i,Jmax+1-j,k)
        ENDDO
        DO i=Imax+1,Imx
           fn(i,k)=0.0_r8
           fs(i,k)=0.0_r8
        ENDDO
     ENDDO

     fna=RESHAPE(fn,(/Imx*Ldim/))
     gna=RESHAPE(gn,(/Imx*Ldim/))
     fsa=RESHAPE(fs,(/Imx*Ldim/))
     gsa=RESHAPE(gs,(/Imx*Ldim/))
     CALL fft991 (fna, gna, inc, Imx, Imax, Ldim, ICaseRec)
     CALL fft991 (fsa, gsa, inc, Imx, Imax, Ldim, ICaseRec)
     fn=RESHAPE(fna,(/Imx,Ldim/))
     fs=RESHAPE(fsa,(/Imx,Ldim/))
     CALL SymAsy (fn, fs, Ldim)
     CALL FourierToSpecCoef (fn, fs, qf, Ldim, j)

  ENDDO

END SUBROUTINE DectoSpherHarm


SUBROUTINE UVtoDivgVort (u, v, qdiv, qrot)

  IMPLICIT NONE

  REAL (KIND=r8), DIMENSION (Imax,Jmax,Kmax), INTENT (IN)  :: u, v
  REAL (KIND=r8), DIMENSION (Mnwv2,Kmax)    , INTENT (OUT) :: qrot, qdiv

  INTEGER :: i, j, k, inc

  REAL (KIND=r8), DIMENSION (Jmaxhf) :: coslat

  REAL (KIND=r8), DIMENSION (Imx,Kmax) :: un, us, vn, vs, gw
  REAL (KIND=r8), DIMENSION (Imx*Kmax) :: una, usa, vna, vsa, gwa

  gw=0.0_r8
  qrot=0.0_r8
  qdiv=0.0_r8
  inc=1

  coslat=SIN(colrad)
  DO j=1,Jmaxhf
     DO k=1,Kmax
        DO i=1,Imax
           un(i,k)=u(i,j,k)*coslat(j)
           us(i,k)=u(i,Jmax+1-j,k)*coslat(j)
           vn(i,k)=v(i,j,k)*coslat(j)
           vs(i,k)=v(i,Jmax+1-j,k)*coslat(j)
        ENDDO
        DO i=Imax+1,Imx
           un(i,k)=0.0_r8
           us(i,k)=0.0_r8
           vn(i,k)=0.0_r8
           vs(i,k)=0.0_r8
        ENDDO
     ENDDO
     una=RESHAPE(un,(/Imx*Kmax/))
     usa=RESHAPE(us,(/Imx*Kmax/))
     vna=RESHAPE(vn,(/Imx*Kmax/))
     vsa=RESHAPE(vs,(/Imx*Kmax/))
     gwa=RESHAPE(gw,(/Imx*Kmax/))
     CALL fft991 (una, gwa, inc, Imx, Imax, Kmax, ICaseRec)
     CALL fft991 (usa, gwa, inc, Imx, Imax, Kmax, ICaseRec)
     CALL fft991 (vna, gwa, inc, Imx, Imax, Kmax, ICaseRec)
     CALL fft991 (vsa, gwa, inc, Imx, Imax, Kmax, ICaseRec)
     un=RESHAPE(una,(/Imx,Kmax/))
     us=RESHAPE(usa,(/Imx,Kmax/))
     vn=RESHAPE(vna,(/Imx,Kmax/))
     vs=RESHAPE(vsa,(/Imx,Kmax/))
     CALL SymAsy (un, us, Kmax)
     CALL SymAsy (vn, vs, Kmax)
     CALL GetConv  (us, un, vs, vn, qdiv, j)
     CALL GetVort  (us, un, vs, vn, qrot, j)
  ENDDO
 
! Get_Conv computes convergence, 
!          signal of qdiv must be changed to get divergence

  qdiv=-qdiv

END SUBROUTINE UVtoDivgVort


SUBROUTINE TransSpherHarm (Ldim, qCoef, ICase)

  IMPLICIT NONE

  INTEGER, INTENT (IN) :: Ldim, ICase

  REAL (KIND=r8), DIMENSION (Mnwv2,Ldim), INTENT (IN OUT) :: qCoef

  INTEGER :: k, l, lx, mn, mm, Nmax, nn

  REAL (KIND=r8), DIMENSION (Mnwv2) :: qWork

  qWork=0.0_r8
  IF (ICase == 1) THEN
     DO k=1,Ldim
        l=0
        DO mm=1,Mend1
           Nmax=Mend2-mm
           DO nn=1,Nmax
              l=l+1
              lx=la0(mm,nn)
              qWork(2*l-1)=qCoef(2*lx-1,k)
              qWork(2*l)=qCoef(2*lx,k)
           ENDDO
        ENDDO
        DO mn=1,Mnwv2
           qCoef(mn,k)=qWork(mn)
        ENDDO
     ENDDO
  ELSE
     DO k=1,Ldim
        l=0
        DO mm=1,Mend1
           Nmax=Mend2-mm
           DO nn=1,Nmax
              l=l+1
              lx=la0(mm,nn)
              qWork(2*lx-1)=qCoef(2*l-1,k)
              qWork(2*lx)=qCoef(2*l,k)
           ENDDO
        ENDDO
        DO mn=1,Mnwv2
           qCoef(mn,k)=qWork(mn)
        ENDDO
     ENDDO
  ENDIF

END SUBROUTINE TransSpherHarm


END MODULE Decomposition
