!
!   Jose Paulo Bonatti - DMD/CPTEC - May 2011
!
MODULE InputArrays

  USE InputParameters, ONLY: r4, r8, KmaxInp, ImaxInp, JmaxInp, &
                             ImaxOut, JmaxOut, KmaxOut, KmaxOutp, &
                             NTracers, Mnwv2Out

  IMPLICIT NONE

  PRIVATE

  PUBLIC :: GetArrays, ClsArrays


  REAL (KIND=r4), DIMENSION (:), ALLOCATABLE, PUBLIC :: &
                  SigIOut, SigLOut, HybIOut, HybLOut

  REAL (KIND=r4), DIMENSION (:), ALLOCATABLE, PUBLIC :: WorkSC

  REAL (KIND=r4), DIMENSION (:,:), ALLOCATABLE, PUBLIC :: &
                  WorkInp, WorkOut,gWorkOut

  REAL (KIND=r8), DIMENSION (:), ALLOCATABLE, PUBLIC :: &
                  SigInterOut, SigLayerOut

  REAL (KIND=r8), DIMENSION (:), ALLOCATABLE, PUBLIC :: &
                  TopoSC,TopoNewSC, LnPsSC

  REAL (KIND=r8), DIMENSION (:,:), ALLOCATABLE, PUBLIC :: &
                  TvirSC, SpHuSC, DivgSC, VortSC

  REAL (KIND=r8), DIMENSION (:,:), ALLOCATABLE, PUBLIC :: &
                  TopoInp, TopoPrs,TopoSmt, TopoOut,TopoNewOut, PsnmInp, PsInp, PsOut, LnPsOut

  REAL (KIND=r8), DIMENSION (:,:,:), ALLOCATABLE, PUBLIC :: &
                  TempInp, UVelInp, VVelInp, gPresaux ,&
                  TempOut, UVelOut, VVelOut, PresOut, &
                  TempSig, UVelSig, VVelSig, PresSig

  REAL (KIND=r8), DIMENSION (:,:,:,:), ALLOCATABLE, PUBLIC :: &
                  SpHuInp, SpHuOut, SpHuSig


CONTAINS


SUBROUTINE GetArrays ()

  IMPLICIT NONE

  ALLOCATE (WorkSC (Mnwv2Out))
  ALLOCATE (SigIOut (KmaxOutp), &
            SigLOut (KmaxOut) , &
            HybIOut(KmaxOutp), HybLOut(KmaxOutp))
  ALLOCATE (SigInterOut (KmaxOutp), &
            SigLayerOut (KmaxOut))
!  ALLOCATE (qTopoOut (Mnwv2Out), qLnPsOut (Mnwv2Out))

  ALLOCATE (TopoSC (Mnwv2Out), &
            TopoNewSC (Mnwv2Out), &
            LnPsSC (Mnwv2Out))
  ALLOCATE (TvirSC (Mnwv2Out,KmaxOut), &
            SpHuSC (Mnwv2Out,KmaxOut))
  ALLOCATE (DivgSC (Mnwv2Out,KmaxOut), &
            VortSC (Mnwv2Out,KmaxOut))
  ALLOCATE (WorkInp (ImaxInp,JmaxInp), &
            WorkOut (ImaxOut,JmaxOut))
  ALLOCATE (TopoInp (ImaxInp,JmaxInp), &
            TopoPrs (ImaxOut,JmaxOut), &
            TopoSmt (ImaxOut,JmaxOut), &
            TopoOut (ImaxOut,JmaxOut) ,&
            TopoNewOut (ImaxOut,JmaxOut))
  ALLOCATE (PsnmInp (ImaxInp,JmaxInp), &
            PsInp (ImaxOut,JmaxOut), &
            PsOut (ImaxOut,JmaxOut), &
            LnPsOut (ImaxOut,JmaxOut))
  ALLOCATE (TempInp (ImaxInp,JmaxInp,KmaxInp), &
            UvelInp (ImaxInp,JmaxInp,KmaxInp), &
            VvelInp (ImaxInp,JmaxInp,KmaxInp))
  ALLOCATE (TempOut (ImaxOut,JmaxOut,KmaxInp), &
            UvelOut (ImaxOut,JmaxOut,KmaxInp), &
            VvelOut (ImaxOut,JmaxOut,KmaxInp), &
            PresOut (ImaxOut,JmaxOut,KmaxInp),&
            gWorkOut(ImaxOut,JmaxOut))
  ALLOCATE (TempSig (ImaxOut,JmaxOut,KmaxOut), &
            UvelSig (ImaxOut,JmaxOut,KmaxOut), &
            VvelSig (ImaxOut,JmaxOut,KmaxOut), &
            gPresaux (ImaxOut,JmaxOut,KmaxOut+1), &
            PresSig (ImaxOut,JmaxOut,KmaxOut))
  ALLOCATE (SpHuInp (ImaxInp,JmaxInp,KmaxInp,NTracers))
  ALLOCATE (SpHuOut (ImaxOut,JmaxOut,KmaxInp,NTracers))
  ALLOCATE (SpHuSig (ImaxOut,JmaxOut,KmaxOut,NTracers))


END SUBROUTINE GetArrays


SUBROUTINE ClsArrays ()

  IMPLICIT NONE

  DEALLOCATE (WorkSC)
  DEALLOCATE (SigIOut, SigLOut)
  DEALLOCATE (SigInterOut, SigLayerOut)
  DEALLOCATE (TopoSC,TopoNewSC, LnPsSC)
  DEALLOCATE (TvirSC, SpHuSC, DivgSC, VortSC)
  DEALLOCATE (WorkInp, WorkOut)
  DEALLOCATE (TopoInp, TopoPrs,TopoSmt, TopoOut,TopoNewOut, PsInp, PsOut, LnPsOut)
  DEALLOCATE (TempInp, UvelInp, VvelInp)
  DEALLOCATE (TempOut, UvelOut, VvelOut, PresOut)
  DEALLOCATE (TempSig, UvelSig, VvelSig, PresSig,gPresaux)
  DEALLOCATE (SpHuInp, SpHuOut, SpHuSig)

END SUBROUTINE ClsArrays


END MODULE InputArrays
