!
!   Jose Paulo Bonatti - DMD/CPTEC - May 2011
!
MODULE InputParameters

  IMPLICIT NONE

  PRIVATE

  PUBLIC :: InitParameters

  ! SELECTED_INT_KIND(R):
  !   The value of the result is the kind type parameter value 
  !   of the integer type that can represent all integer values 
  !   n in the range -10**R < n < 10**R
  !   R must be a scalar of integer type(kind default)

  ! SELECTED_REAL_KIND(P,R):
  !   The value of the result is the kind type parameter value 
  !   of the real type that has a decimal precision greater than 
  !   or equal to P digits as returned by the function PRECISION 
  !   and a decimal exponent range greater than or equal to R 
  !   as returned by the function RANGE
  !   P (optional) must be a scalar of integer (kind default)
  !   R (optional) must be a scalar of integer (kind default)

  INTEGER, PARAMETER, PUBLIC :: i4=SELECTED_INT_KIND(9)
  INTEGER, PARAMETER, PUBLIC :: r4=SELECTED_REAL_KIND(6)
  INTEGER, PARAMETER, PUBLIC :: r8=SELECTED_REAL_KIND(15)

  INTEGER, PUBLIC :: ImaxInp, JmaxInp, KmaxInp, KmaxInq, &
                     MendOut, ImaxOut, JmaxOut, KmaxOut, MendMin, MendCut

  INTEGER, PUBLIC :: Mend1Out, Mend2Out, Mend3Out, &
                     Mnwv2Out, Mnwv3Out, Mnwv0Out, Mnwv1Out, &
                     ImxOut,  JmaxhfOut, KmaxOutp, &
                     NTracers, Kdim, ICaseDec, ICaseRec, &
                     MFactorFourier, MTrigsFourier,Iter

  INTEGER, PUBLIC :: nferr, nfinp, nfprt, nfgrd, nfcft, nfprs, nfsig,nftop,nfctl

  INTEGER (KIND=i4), PUBLIC :: ForecastDay

  REAL (KIND=r4), PUBLIC :: TimeOfDay

  REAL (KIND=r8), PUBLIC :: cTv

  REAL (KIND=r8), PUBLIC :: SmthPerCut

  CHARACTER (LEN=526), PUBLIC :: DataInp, DataOut,DataTop, DataTopGrid
  CHARACTER (LEN=1024), PUBLIC :: DirTop
  CHARACTER (LEN=1024), PUBLIC :: DirICnInp, DirICnOut

  REAL (KIND=r8), PUBLIC :: EMRad ! Earth Mean Radius (m)
  REAL (KIND=r8), PUBLIC :: EMRad1 ! 1/EMRad (1/m)
  REAL (KIND=r8), PUBLIC :: EMRad12 ! EMRad1**2 (1/m2)
  REAL (KIND=r8), PUBLIC :: EMRad2 ! EMRad**2 (m2)
  REAL (KIND=r8), PUBLIC :: Grav ! Gravity Acceleration (m2/s)
  REAL (KIND=r8), PUBLIC :: Rd ! Dry Air Gas Constant (m2/s2/K)
  REAL (KIND=r8), PUBLIC :: Rv ! Water Vapour Air Gas Constant (m2/s2/K)
  REAL (KIND=r8), PUBLIC :: Cp ! Dry Air Heat Capacity (m2/s2/K)
  REAL (KIND=r8), PUBLIC :: Lc ! Latent Heat of Condensation (m2/s2)
  REAL (KIND=r8), PUBLIC :: Gamma ! Mean Atmospheric Lapse Rate (K/m)
  REAL (KIND=r8), PUBLIC :: GEps ! Precision For Constante Lapse Rate (No Dim)
  REAL (KIND=r8), PUBLIC :: RoCp    ! Rd over Cp
  REAL (KIND=r8), PUBLIC :: RoCp1   ! RoCp + 1
  REAL (KIND=r8), PUBLIC :: RoCpr   ! 1 / RoCp

  LOGICAL, PUBLIC :: GrADS, GrADSOnly,SmoothTopo

  REAL (KIND=r8), PUBLIC :: Undef, Lon0, Lat0, Cte

  CHARACTER (LEN=3), PUBLIC :: Case

  CHARACTER (LEN=10), PUBLIC :: TruncOut, DateICn,DateICn_previous

  CHARACTER (LEN=12), PUBLIC :: TimeGr

  CHARACTER (LEN=3), DIMENSION (12), PUBLIC :: MonChar

  REAL (KIND=r8), DIMENSION (:), ALLOCATABLE, PUBLIC :: PresInp, DelSigmaOut

  REAL (KIND=r8), DIMENSION (:), ALLOCATABLE, PUBLIC ::a_hybr, b_hybr

  LOGICAL, PUBLIC :: PolarMean, LinearInterp

  LOGICAL, PUBLIC :: FlagInput(5), FlagOutput(5)

  INTEGER (KIND=i4), PUBLIC :: DateInp(4), DateCur(4)

  INTEGER, DIMENSION (:,:), ALLOCATABLE, PUBLIC :: MaskInput

  INTEGER (KIND=i4) :: Year, Month, Day, Hour

  LOGICAL ,PUBLIC :: SigmaLevel=.false.

CONTAINS


SUBROUTINE InitParameters ()

  IMPLICIT NONE

  CHARACTER (LEN=1) :: Grid
  CHARACTER (LEN=15) :: DataSig
  CHARACTER (LEN=17) :: DataHyb
  CHARACTER (LEN=21) :: DataPrs
  CHARACTER (LEN=1024) :: DirVert
  CHARACTER (LEN=1024) :: DirMainInp, DirMainOut
  INTEGER :: k
  NAMELIST /ICnAGCMNML/ MendOut, KmaxOut, DateICn,DateICn_previous, MendMin, MendCut, Iter,SmthPerCut,SmoothTopo, &
                        Case, Grid, SigmaLevel,GrADS, GrADSOnly,DirMainInp, DirMainOut

  MonChar=(/ 'jan', 'feb', 'mar', 'apr', 'may', 'jun', &
             'jul', 'aug', 'sep', 'oct', 'nov', 'dec' /)

  ! Files Units

  nferr=0    ! Standard Error Print Out
  nfinp=5    ! Standard Read In
  nfprt=6    ! Standard Print Out
  nfgrd=10   ! To Read Gridded Initial Conditions in Data Resolution
  nftop=50   ! To Read Gridded topography in Data Resolution
  nfcft=20   ! To Write AGCM Initial Conditions In CPTEC Format
  nfprs=30   ! To Read Data Pressure Levels
  nfsig=40   ! To Read Delta Sigma for Selected Vertical Resolution
  nfctl=75   ! Unit To Write Control File For GrADS
  MendMin=62     ! Minimum Spectral Resolution For Doing Topography Smoothing
  MendCut=0       ! Spectral Resolution Cut Off for Topography Smoothing
  Iter=10         ! Number of Iteractions in Topography Smoothing
  SmthPerCut=0.12_r8 ! Percentage for Topography Smoothing

  MendOut=62  ! Spectral Horizontal Resolution of Output Data
  KmaxOut=28  ! Number of Layers of Output Data
  GrADS=.TRUE.       ! Flag to Produce GrADS Files
  GrADSOnly=.FALSE.  ! Flag to Only Produce GrADS Files (Do Not Produce Inputs for Model)
  SmoothTopo=.TRUE.  ! Flag to Performe Topography Smoothing
  DateICn='yyyymmddhh' ! Date: yyyymmddhh
  DateICn_previous='yyyymmddhh' ! DateICn_previous
  Case='E40'  ! Case: Era 40 at CPTEC (E40) or ERA 40 at WEB of ECMWF (E4W) or 
              ! Era Five  at WEB of ECMWF (ER5) or 
              ! Era Interim  at WEB of ECMWF (EIT) or 
              ! NCEP/NCAR Analysis at WEB of NCAR (FNL) Data
  Grid='Q'    ! Grid: Quadratic (Q) or Linear (L)
  DirMainInp='./ ' ! Main Directory for Datain
  DirMainOut='./ ' ! Main Directory for Dataout

  READ (UNIT=nfinp, NML=ICnAGCMNML)

  READ (DateICn(1:4),  '(I4)') Year
  READ (DateICn(5:6),  '(I2)') Month
  READ (DateICn(7:8),  '(I2)') Day
  READ (DateICn(9:10), '(I2)') Hour
  TimeGr='  z         '
  TimeGr(1:2)=DateICn(9:10)    ! Hour of the Day
  TimeGr(4:5)=DateICn(7:8)     ! Day of the Month
  TimeGr(6:8)=MonChar(Month) ! Month of the Year
  TimeGr(9:12)=DateICn(1:4)    ! Year
  DateInp(1)=Hour
  DateInp(2)=Month
  DateInp(3)=Day
  DateInp(4)=Year
  DateCur=DateInp

  SELECT CASE (Case)
     CASE ('E40')
        ImaxInp=320 ! Number of Longitudes of Input Data
        JmaxInp=161 ! Number of Latitudes of Input Data
        KmaxInp=23  ! Number of Layers of Input Data
        KmaxInq=23  ! Number of Layers of Input Data for Humidity
        DataPrs='PressureLevels.ERA40C'
        DataInp='ERA40CATM'//DateICn//'.dat'
     CASE ('E4W')
        ImaxInp=144 ! Number of Longitudes of Input Data
        JmaxInp=73  ! Number of Latitudes of Input Data
        KmaxInp=23  ! Number of Layers of Input Data
        KmaxInq=23  ! Number of Layers of Input Data for Humidity
        DataPrs='PressureLevels.ERA40W'
        DataInp='ERA40WATM'//DateICn//'.dat'
     CASE ('EIT')
        ImaxInp=240 ! Number of Longitudes of Input Data
        JmaxInp=121 ! Number of Latitudes of Input Data
        KmaxInp=37  ! Number of Layers of Input Data
        KmaxInq=37  ! Number of Layers of Input Data for Humidity
        DataPrs='PressureLevels.ERAINT'
        DataInp='ERAINTATM'//DateICn//'.dat'
     CASE ('ER5')
        ImaxInp=1440 ! Number of Longitudes of Input Data
        JmaxInp=721 ! Number of Latitudes of Input Data
        KmaxInp=37  ! Number of Layers of Input Data
        KmaxInq=37  ! Number of Layers of Input Data for Humidity
        DataPrs='PressureLevels.ERA5'
        DataInp='ERA5ATM'//DateICn//'.dat'
     CASE ('FNL')
        ImaxInp=360 ! Number of Longitudes of Input Data
        JmaxInp=181 ! Number of Latitudes of Input Data
        KmaxInp=26  ! Number of Layers of Input Data
        KmaxInq=21  ! Number of Layers of Input Data for Humidity
        DataPrs='PressureLevels.NMCFNL'
        DataInp='NMCFNLATM'//DateICn//'.dat'
     CASE ('GFS')
        ImaxInp=1440 ! Number of Longitudes of Input Data
        JmaxInp=721 ! Number of Latitudes of Input Data
        KmaxInp=26  ! Number of Layers of Input Data
        KmaxInq=21  ! Number of Layers of Input Data for Humidity
        DataPrs='PressureLevels.NMCGFS'
        DataInp='NMCGFSATM'//DateICn//'.dat'
     CASE ('CPT')
        ImaxInp=192 ! Number of Longitudes of Input Data
        JmaxInp=96 ! Number of Latitudes of Input Data
        KmaxInp=28  ! Number of Layers of Input Data
        KmaxInq=28  ! Number of Layers of Input Data for Humidity
        DataPrs='PressureLevels.NMCCPT'
        DataInp='GPRGNMC'//TRIM(DateICn_previous)//''//TRIM(DateICn)//'F.fct.TQ0062L028'     !NMCGFSATM'//DateICn//'.dat'
  END SELECT

  ALLOCATE (PresInp (KmaxInp))

  ALLOCATE (DelSigmaOut (KmaxOut))
  ALLOCATE (a_hybr(KmaxOut+1))
  ALLOCATE (b_hybr(KmaxOut+1))
  
  DirICnInp=TRIM(DirMainInp)//'DataAnl/'//Case//'/ '
  DirICnOut=TRIM(DirMainOut)//'model/datain/ '
  DirVert=TRIM(DirMainInp)//'/DataLevels/ '
  DirTop=TRIM(DirMainInp)//'/DataTop/'
  DataSig='DeltaSigma.L   '
  DataHyb='HybridLevels.L   '
  DataTop='Topography.TQ'
  DataTopGrid='Topography.G00096'

  WRITE (DataSig(13:15), FMT='(I3.3)') KmaxOut
  WRITE (DataHyb(15:17), FMT='(I3.3)') KmaxOut

  CALL GetImaxJmax (MendOut, ImaxOut, JmaxOut)

  IF (ImaxInp > ImaxOut) THEN
     LinearInterp=.FALSE.
  ELSE
     LinearInterp=.TRUE.
  END IF

  WRITE (UNIT=nfprt, FMT='(A)')      ' '
  WRITE (UNIT=nfprt, FMT='(A)')      ' &ICNAGCMNML'
  WRITE (UNIT=nfprt, FMT='(A,I5)')   '       IMaxInp = ', IMaxInp
  WRITE (UNIT=nfprt, FMT='(A,I5)')   '       JMaxInp = ', JMaxInp
  WRITE (UNIT=nfprt, FMT='(A,I5)')   '       KmaxInp = ', KmaxInp
  WRITE (UNIT=nfprt, FMT='(A,I5)')   '       KmaxInq = ', KmaxInq
  WRITE (UNIT=nfprt, FMT='(A,I5)')   '       IMaxOut = ', IMaxOut
  WRITE (UNIT=nfprt, FMT='(A,I5)')   '       JMaxOut = ', JMaxOut
  WRITE (UNIT=nfprt, FMT='(A,I5)')   '       MendOut = ', MendOut
  WRITE (UNIT=nfprt, FMT='(A,I5)')   '       KmaxOut = ', KmaxOut
  WRITE (UNIT=nfprt, FMT='(2A)')     '       DateICn = ', DateICn
  WRITE (UNIT=nfprt, FMT='(2A)')     '          Case = ', Case
  WRITE (UNIT=nfprt, FMT='(2A)')     '          Grid = ', Grid
  WRITE (UNIT=nfprt, FMT='(A,L)')    '  LinearInterp = ', LinearInterp
  WRITE (UNIT=nfprt, FMT='(2A)')     '    DirMainInp = ', TRIM(DirMainInp)
  WRITE (UNIT=nfprt, FMT='(2A)')     '    DirMainOut = ', TRIM(DirMainOut)
  WRITE (UNIT=nfprt, FMT='(2A)')     '     DirICnInp = ', TRIM(DirICnInp)
  WRITE (UNIT=nfprt, FMT='(2A)')     '     DirICnOut = ', TRIM(DirICnOut)
  WRITE (UNIT=nfprt, FMT='(2A)')     '       DirVert = ', TRIM(DirVert)
  WRITE (UNIT=nfprt, FMT='(2A)')     '        DirTop = ', TRIM(DirTop)
  WRITE (UNIT=nfprt, FMT='(2A)')     '       DataPrs = ', TRIM(DataPrs)
  WRITE (UNIT=nfprt, FMT='(2A)')     '       DataSig = ', TRIM(DataSig)
  WRITE (UNIT=nfprt, FMT='(2A)')     '       DataSig = ', TRIM(DataHyb)
  WRITE (UNIT=nfprt, FMT='(2A)')     '   DataTopGrid = ', TRIM(DataTopGrid)  
  
  WRITE (UNIT=nfprt, FMT='(1A)')     '  Flag to Performe Topography Smoothing'
  WRITE (UNIT=nfprt, FMT='(A,L,A)')  '  SmoothTopo   =',SmoothTopo ,'! Flag to Performe Topography Smoothing'
  WRITE (UNIT=nfprt, FMT='(A,I5,A)') '  MendMin      =',MendMin    ,'! Minimum Spectral Resolution For Doing Topography Smoothing'
  WRITE (UNIT=nfprt, FMT='(A,I5,A)') '  MendCut      =',MendCut    ,'! Spectral Resolution Cut Off for Topography Smoothing'
  WRITE (UNIT=nfprt, FMT='(A,I5,A)') '  Iter         =',Iter       ,'! Number of Iteractions in Topography Smoothing'
  WRITE (UNIT=nfprt, FMT='(A,F5.2,A)')' SmthPerCut   =',SmthPerCut ,'! Percentage for Topography Smoothing'
  WRITE (UNIT=nfprt, FMT='(A)')      ' /'

  IF(TRIM(Case) == 'CPT')THEN
     OPEN  (UNIT=nfprs, FILE=TRIM(DirVert)//TRIM(DataPrs), STATUS='OLD')
     READ  (UNIT=nfprs, FMT='(8F14.7)') PresInp
     CLOSE (UNIT=nfprs)
  ELSE
     OPEN  (UNIT=nfprs, FILE=TRIM(DirVert)//TRIM(DataPrs), STATUS='OLD')
     READ  (UNIT=nfprs, FMT='(8F7.1)') PresInp
     CLOSE (UNIT=nfprs)
  END IF

  OPEN  (UNIT=nfsig, FILE=TRIM(DirVert)//TRIM(DataSig), STATUS='OLD')
  READ  (UNIT=nfsig, FMT='(5F9.6)') DelSigmaOut
  CLOSE (UNIT=nfsig)

  OPEN  (UNIT=nfsig, FILE=TRIM(DirVert)//TRIM(DataHyb), STATUS='OLD')
  DO k=1,KmaxOut+1
      READ(UNIT=nfsig,FMT=*) a_hybr(k), b_hybr(k)
      a_hybr(k) = a_hybr(k) / 100._r4  ! transform from Pa to mbar
      WRITE(unit=nfprt,FMT=*) a_hybr(k), b_hybr(k)
  ENDDO

  CLOSE (UNIT=nfsig)

  WRITE (UNIT=nfprt, FMT='(/,A)')      '  PresInp:'
  WRITE (UNIT=nfprt, FMT='(8F7.1)')     PresInp
  WRITE (UNIT=nfprt, FMT='(/,A)')      '  DelSigmaOut:'
  WRITE (UNIT=nfprt, FMT='(4E15.8)')    DelSigmaOut
  WRITE (UNIT=nfprt, FMT='(/,A)')      '  a_hybr(k)'
  WRITE (UNIT=nfprt, FMT='(4E15.8)')    a_hybr
  WRITE (UNIT=nfprt, FMT='(/,A)')      '  b_hybr(k)'
  WRITE (UNIT=nfprt, FMT='(4E15.8)')    b_hybr

  WRITE (UNIT=nfprt, FMT='(/,A,F16.4,3X,1P,E12.4)') '  SumDelSigmaOut = ', &
         SUM(DelSigmaOut), 1.0_r8-SUM(DelSigmaOut)

  TruncOut='T     L   '
  TruncOut(2:2)=Grid
  WRITE (TruncOut(3:6), '(I4.4)') MendOut
  WRITE (TruncOut(8:10), '(I3.3)') KmaxOut
  WRITE (DataTop(14:17),'(I4.4)')MendOut
  WRITE (DataTopGrid(13:17),'(I5.5)')JmaxOut
  ! CPTEC Analysis File Name
  DataOut='GANL'//Case//DateICn//'S.unf.'//TruncOut 
  SELECT CASE (Case)
    CASE ('CPT')
        DataInp='GPRGNMC'//TRIM(DateICn_previous)//''//TRIM(DateICn)//'F.fct.'//TruncOut     !NMCGFSATM'//DateICn//'.dat'
  END SELECT

  WRITE (UNIT=nfprt, FMT='(A)')    ' '
  WRITE (UNIT=nfprt, FMT='(2A)')   '  DataInp    = ', TRIM(DataInp)
  WRITE (UNIT=nfprt, FMT='(2A)')   '  DataOut    = ', TRIM(DataOut)
  WRITE (UNIT=nfprt, FMT='(2A)')   '  DataTop    = ', TRIM(DataTop)
  WRITE (UNIT=nfprt, FMT='(2A)')   '  DataTopGrid= ', TRIM(DataTopGrid)
  WRITE (UNIT=nfprt, FMT='(/,A)')  '  Numerical Precision (KIND): '
  WRITE (UNIT=nfprt, FMT='(A,I5)') '          i4 = ', i4
  WRITE (UNIT=nfprt, FMT='(A,I5)') '          r4 = ', r4
  WRITE (UNIT=nfprt, FMT='(A,I5)') '          r8 = ', r8

  Mend1Out=MendOut+1
  Mend2Out=MendOut+2
  Mend3Out=MendOut+3
  Mnwv2Out=Mend1Out*Mend2Out
  Mnwv0Out=Mnwv2Out/2
  Mnwv3Out=Mnwv2Out+2*Mend1Out
  Mnwv1Out=Mnwv3Out/2

  ImxOut=ImaxOut+2
  JmaxhfOut=JmaxOut/2
  KmaxOutp=KmaxOut+1

  NTracers=1
  Kdim=1
  ICaseDec=1
  ICaseRec=-1

  MFactorFourier=64
  MTrigsFourier=3*ImaxOut/2

  ForecastDay=0_i4
  TimeOfDay=0.0_r4

  EMRad=6.37E6_r8
  EMRad1=1.0_r8/EMRad
  EMRad12=EMRad1*EMRad1
  EMRad2=EMRad*EMRad

  Grav=9.80665_r8
  Rd=287.05_r8
  Rv=461.50_r8
  Cp=1004.6_r8
  Lc=2.5E6_r8
  Gamma=-6.5E-3_r8
  GEps=1.E-9_r8
  cTv=Rv/Rd-1.0_r8


  RoCp=Rd/Cp
  RoCp1=RoCp+1.0_r8
  RoCpr=1.0_r8/RoCp

  Cte=1.0_r8/SQRT(2.0_r8)

  Undef=-999999.0_r8

  ! For Bi-Linear Interpolation

  Lon0=0.0_r8  ! Start at Prime Meridian
  Lat0=90.0_r8 ! Start at North Pole

  ! For Areal Weighted Interpolation

  ALLOCATE (MaskInput(IMaxInp,JMaxInp))
  MaskInput=1

  PolarMean=.FALSE.

  FlagInput(1)=.TRUE.  ! Start at North Pole
  FlagInput(2)=.TRUE.  ! Start at Prime Meridian
  FlagInput(3)=.TRUE.  ! Latitudes Are at Center of Box
  FlagInput(4)=.TRUE.  ! Longitudes Are at Center of Box
  FlagInput(5)=.FALSE. ! Regular Grid

  FlagOutput(1)=.TRUE.  ! Start at North Pole
  FlagOutput(2)=.TRUE.  ! Start at Prime Meridian
  FlagOutput(3)=.FALSE. ! Latitudes Are at North Edge of Box
  FlagOutput(4)=.TRUE.  ! Longitudes Are at Center of Box
  FlagOutput(5)=.TRUE.  ! Gaussian Grid


END SUBROUTINE InitParameters


SUBROUTINE GetImaxJmax (Mend, Imax, Jmax)

  IMPLICIT NONE

  INTEGER, INTENT (IN) :: Mend
  INTEGER, INTENT (OUT) :: Imax, Jmax

  INTEGER :: Nx, Nm, N2m, N3m, N5m, &
             n2, n3, n5, j, n, Check, Jfft

  INTEGER, SAVE :: Lfft=40000

  INTEGER, DIMENSION (:), ALLOCATABLE, SAVE :: Ifft

  N2m=CEILING(LOG(REAL(Lfft,r8))/LOG(2.0_r8))
  N3m=CEILING(LOG(REAL(Lfft,r8))/LOG(3.0_r8))
  N5m=CEILING(LOG(REAL(Lfft,r8))/LOG(5.0_r8))
  Nx=N2m*(N3m+1)*(N5m+1)

  ALLOCATE (Ifft (Nx))
  Ifft=0

  n=0
  DO n2=1,N2m
     Jfft=(2**n2)
     IF (Jfft > Lfft) EXIT
     DO n3=0,N3m
        Jfft=(2**n2)*(3**n3)
        IF (Jfft > Lfft) EXIT
        DO n5=0,N5m
           Jfft=(2**n2)*(3**n3)*(5**n5)
           IF (Jfft > Lfft) EXIT
           n=n+1
           Ifft(n)=Jfft
        END DO
     END DO
  END DO
  Nm=n

  n=0
  DO 
     Check=0
     n=n+1
     DO j=1,Nm-1
        IF (Ifft(j) > Ifft(j+1)) THEN
           Jfft=Ifft(j)
           Ifft(j)=Ifft(j+1)
           Ifft(j+1)=Jfft
           Check=1
        END IF
     END DO
     IF (Check == 0) EXIT
  END DO

  Imax=3*Mend+1
  DO n=1,Nm
     IF (Ifft(n) >= Imax) THEN
        Imax=Ifft(n)
        EXIT
     END IF
  END DO
  Jmax=Imax/2
  IF (MOD(Jmax, 2) /= 0) Jmax=Jmax+1

  DEALLOCATE (Ifft)

END SUBROUTINE GetImaxJmax


END MODULE InputParameters
