!
!   Jose Paulo Bonatti - DMD/CPTEC - May 2011
!
PROGRAM SpectralIntialConditions

   USE InputParameters, ONLY : r4, r8, ImaxInp, JmaxInp, KmaxInp, KmaxInq,GrADS, GrADSOnly,Case, &
                               MendOut,ImaxOut, JmaxOut, KmaxOut, KmaxOutp,Iter, MendCut,SmthPerCut,SmoothTopo, &
                               Mnwv2Out, Kdim, NTracers, ICaseDec,SigmaLevel,MonChar, &
                               nfprt, nfgrd, nftop,nfcft ,RoCp1,RoCpR,nfctl,nferr,&
                               ForecastDay, TimeOfDay, cTv, Cte, LinearInterp, &
                               DirTop, DirICnInp, DirICnOut, DataInp, DataOut,DataTop, &
                               PresInp, DelSigmaOut, DateInp, DateCur,DataTopGrid, &
                               a_hybr, b_hybr,TruncOut, InitParameters

   USE InputArrays, ONLY : SigLayerOut, SigInterOut, SigIOut, SigLOut,gWorkOut, &
                           TopoInp, PsnmInp, PsInp, TopoPrs, TopoSmt,TopoOut, PsOut, LnPsOut, &
                           TempInp, UVelInp, VVelInp, SpHuInp, gPresaux,&
                           TempOut, UVelOut, VVelOut, SpHuOut, PresOut, &
                           TempSig, UVelSig, VVelSig, SpHuSig, PresSig, TopoNewSC,&
                           TopoSC, LnPsSC, TvirSC, SpHuSC, DivgSC, VortSC, TopoNewOut,&
                           WorkInp, WorkOut, WorkSC, GetArrays, ClsArrays,HybIOut, HybLOut

   USE LinearInterpolation, ONLY : InitLinearInterpolation, DoLinearInterpolation

   USE AreaInterpolation, ONLY : InitAreaInterpolation, DoAreaInterpolation

   USE VerticalInterpolation, ONLY : VertSigmaInter

   USE Recomposition, ONLY: InitRecomposition, ClsMemRecomposition, &
       RecompositionScalar, RecompositionVector, &
       DivgVortToUV, coslat

   USE Utils, ONLY : NewPs, NewSigma, MoistVertExtrap

   USE Fourier, ONLY : InitFFT, ClsMemFFT

   USE Legendre, ONLY : InitLegendre, ClsMemLegendre,glat

   USE Decomposition, ONLY : DectoSpherHarm, UVtoDivgVort, TransSpherHarm

   IMPLICIT NONE

   INTEGER :: i, j, k, LRec, nRec,ios,IOL
   INTEGER :: IDSLInp=2
   LOGICAL :: GetNewTop=.FALSE.,OnlyTopo=.FALSE.
   CHARACTER (LEN=12) :: Tdef='  z         '
   
   CALL InitParameters ()

   CALL GetArrays ()

   IF (LinearInterp) THEN
      CALL InitLinearInterpolation ()
   ELSE
      CALL InitAreaInterpolation ()
   END IF

   CALL InitFFT ()

   CALL InitLegendre ()

   INQUIRE (FILE=TRIM(DirTop)//TRIM(DataTop), EXIST=GetNewTop)
   IF (GetNewTop) THEN
      WRITE (UNIT=6, FMT='(/,A)')' Getting New Spec Topography'
      OPEN (UNIT=nftop, FILE=TRIM(DirTop)//TRIM(DataTop), FORM='UNFORMATTED', &
            ACCESS='SEQUENTIAL', ACTION='READ', STATUS='OLD', IOSTAT=ios)
       IF (ios /= 0) THEN
          WRITE (UNIT=6, FMT='(3A,I4)') ' ** (Error) ** Open file ', &
               TRIM(TRIM(DirTop)//TRIM(DataTop)), &
               ' returned IOStat = ', ios
          STOP ' ** (Error) **'
       END IF
       READ  (UNIT=nftop) WorkSC
       CLOSE (UNIT=nftop)
       TopoNewSC=WorkSC



       INQUIRE(IOLENGTH=lrec)WorkOut
       INQUIRE (FILE=TRIM(DirTop)//TRIM(DataTopGrid), EXIST=GetNewTop)
       OPEN (UNIT=nftop, FILE=TRIM(DirTop)//TRIM(DataTopGrid), FORM='UNFORMATTED', &
            ACCESS='DIRECT', ACTION='READ',recl=lrec, STATUS='OLD', IOSTAT=ios)
       IF (ios /= 0) THEN
          WRITE (UNIT=6, FMT='(3A,I4)') ' ** (Error) ** Open file ', &
               TRIM(TRIM(DirTop)//TRIM(DataTopGrid)), &
               ' returned IOStat = ', ios
          STOP ' ** (Error) **'
       END IF
       READ  (UNIT=nftop,rec=1) WorkOut
       CLOSE (UNIT=nftop)
       DO j=1,JmaxOut
          DO I=1,ImaxOut
             TopoNewOut(i,JmaxOut-j+1)=WorkOut(i,j)
          END DO
       END DO
   ELSE
       WRITE (UNIT=6, FMT='(/,A)')' Original Topography'   
   END IF

   IF(TRIM(Case) == 'CPT')THEN
      OPEN (UNIT=nfgrd, FILE=TRIM(DirICnInp)//TRIM(DataInp), STATUS='OLD', &
                     FORM='UNFORMATTED', ACCESS='SEQUENTIAL')

      nRec=1
      READ (UNIT=nfgrd) WorkInp
      TopoInp=REAL(WorkInp,r8)
      TopoOut=TopoInp

      nRec=nRec+1
      READ (UNIT=nfgrd) WorkInp
      PsnmInp=REAL(WorkInp,r8)
      PsInp=PsnmInp

      DO k=1,KmaxInp
         nRec=nRec+1
         READ (UNIT=nfgrd) WorkInp
         TempInp(:,:,k)=REAL(WorkInp,r8)
         TempOut(:,:,k)=TempInp(:,:,k)
         PRINT*,k,MAXVAL(TempOut(:,:,k)),MINVAL(TempOut(:,:,k))
      END DO
      DO k=1,KmaxInp
         nRec=nRec+1
         READ (UNIT=nfgrd) WorkInp
         UVelInp(:,:,k)=REAL(WorkInp,r8)
         UVelOut(:,:,k)=UVelInp(:,:,k)
      END DO
      DO k=1,KmaxInp
         nRec=nRec+1
         READ (UNIT=nfgrd) WorkInp
         VVelInp(:,:,k)=REAL(WorkInp,r8)
         VVelOut(:,:,k)=VVelInp(:,:,k)
      END DO
      DO k=1,KmaxInq
         nRec=nRec+1
         READ (UNIT=nfgrd) WorkInp
         SpHuInp(:,:,k,1)=REAL(WorkInp,r8)
         SpHuOut(:,:,k,1)=SpHuInp(:,:,k,1)
      END DO
      CLOSE (UNIT=nfgrd)
      IF (KmaxInq < KmaxInp) THEN
         CALL MoistVertExtrap (ImaxInp, JmaxInp, KmaxInp, KmaxInq, &
                               PresInp, TempInp, SpHuInp(:,:,:,1))
      END IF
      DO k=1,KmaxInp
         SpHuOut(:,:,k,1)=SpHuInp(:,:,k,1)
      END DO
      TempOut(:,:,:)=TempOut(:,:,:)*(1.0_r8+cTv*SpHuOut(:,:,:,1))

   ELSE 
      INQUIRE (IOLENGTH=LRec) WorkInp
      OPEN (UNIT=nfgrd, FILE=TRIM(DirICnInp)//TRIM(DataInp), STATUS='OLD', &
                     FORM='UNFORMATTED', ACCESS='DIRECT', RECL=LRec)

      nRec=1
      READ (UNIT=nfgrd, REC=nRec) WorkInp
      TopoInp(:,JmaxInp:1:-1)=REAL(WorkInp,r8)
      IF (LinearInterp) THEN
         CALL DoLinearInterpolation (TopoInp, TopoOut)
      ELSE
         CALL DoAreaInterpolation (TopoInp, TopoOut)
      END IF
      nRec=nRec+1
      READ (UNIT=nfgrd, REC=nRec) WorkInp
      PsnmInp(1:ImaxInp,JmaxInp:1:-1)=REAL(WorkInp(1:ImaxInp,1:JmaxInp),r8)
      IF (LinearInterp) THEN
         CALL DoLinearInterpolation (PsnmInp, PsInp)
      ELSE
         CALL DoAreaInterpolation (PsnmInp, PsInp)
      END IF
      DO k=1,KmaxInp
         nRec=nRec+1
         READ (UNIT=nfgrd, REC=nRec) WorkInp
         TempInp(:,JmaxInp:1:-1,k)=REAL(WorkInp,r8)
         IF (LinearInterp) THEN
            CALL DoLinearInterpolation (TempInp(:,:,k), TempOut(:,:,k))
         ELSE
            CALL DoAreaInterpolation (TempInp(:,:,k), TempOut(:,:,k))
         END IF
      END DO
      DO k=1,KmaxInp
         nRec=nRec+1
         READ (UNIT=nfgrd, REC=nRec) WorkInp
         UVelInp(:,JmaxInp:1:-1,k)=REAL(WorkInp,r8)
         IF (LinearInterp) THEN
            CALL DoLinearInterpolation (UVelInp(:,:,k), UVelOut(:,:,k))
         ELSE
            CALL DoAreaInterpolation (UVelInp(:,:,k), UVelOut(:,:,k))
         END IF
      END DO
      DO k=1,KmaxInp
         nRec=nRec+1
         READ (UNIT=nfgrd, REC=nRec) WorkInp
         VVelInp(:,JmaxInp:1:-1,k)=REAL(WorkInp,r8)
         IF (LinearInterp) THEN
            CALL DoLinearInterpolation (VVelInp(:,:,k), VVelOut(:,:,k))
         ELSE
            CALL DoAreaInterpolation (VVelInp(:,:,k), VVelOut(:,:,k))
         END IF
      END DO
      DO k=1,KmaxInq
         nRec=nRec+1
         READ (UNIT=nfgrd, REC=nRec) WorkInp
         SpHuInp(:,JmaxInp:1:-1,k,1)=REAL(WorkInp,r8)
      END DO
      CLOSE (UNIT=nfgrd)
      IF (KmaxInq < KmaxInp) THEN
         CALL MoistVertExtrap (ImaxInp, JmaxInp, KmaxInp, KmaxInq, &
                               PresInp, TempInp, SpHuInp(:,:,:,1))
      END IF
      DO k=1,KmaxInp
         IF (LinearInterp) THEN
            CALL DoLinearInterpolation (SpHuInp(:,:,k,1), SpHuOut(:,:,k,1))
         ELSE
            CALL DoAreaInterpolation (SpHuInp(:,:,k,1), SpHuOut(:,:,k,1))
         END IF
      END DO
      TempOut(:,:,:)=TempOut(:,:,:)*(1.0_r8+cTv*SpHuOut(:,:,:,1))
  END IF

   IF(TRIM(Case) == 'CPT')THEN
      TopoOut=TopoNewOut
   END IF









   TopoPrs=TopoOut
   DO k=1,KmaxInp
      DO j=1,JmaxOut
         DO I=1,ImaxOut
            PresOut(i,j,k)=PresInp(k)
         END DO
      END DO
   END DO

   IF (SigmaLevel ) THEN
      CALL NewSigma (KmaxOut, KmaxOutp, &
                     DelSigmaOut, SigInterOut, SigLayerOut)

      SigIOut=REAL(SigInterOut, r4)
      SigLOut=REAL(SigLayerOut, r4)
   ELSE
      WRITE (UNIT=nfprt, FMT='(/,A)')  ' a_hybr Coef A:'
      WRITE (UNIT=nfprt, FMT='(7F10.6)') a_hybr
      WRITE (UNIT=nfprt, FMT='(/,A)')  ' b_hybr Coef B:'
      WRITE (UNIT=nfprt, FMT='(7F10.6)') b_hybr
   END IF 
  
   WRITE (UNIT=nfprt, FMT='(/,A)') 'Doing Vertical Interpolation'

      ! Calculate pressure for each layer according to vertical coordinate
      IF ( SigmaLevel ) THEN
         ! Sigma case, calculate pressure for each layer directly
         DO j=1,JmaxOut
            DO k=1,KmaxOut
               DO I=1,ImaxOut
                  PresSig(i,j,k)=PsInp(i,j)*SigLayerOut(k)
               END DO
            END DO
         END DO
      ELSE
         ! Sigma-p case, first calculate pressure for each interface and then
         ! calculate pressure on each layer according to vertical structure
         DO j=1,JmaxOut
            DO k=1,KmaxOut+1
               DO I=1,ImaxOut
                  !p(i,k)        = 1.e-2_r8 * a_hybr(k)  + b_hybr(k) * psmb(i,j) 
                  gPresaux(i,j,k)=            a_hybr(k)   + b_hybr(k) * PsInp(i,j)
               END DO
            END DO
         END DO
         IF ( IDSLInp == 2 ) THEN
            ! Mean over two interfaces
            DO j=1,JmaxOut
               DO k=1,KmaxOut
                  DO I=1,ImaxOut
                     PresSig(i,j,k)=(gPresaux(i,j,k)+gPresaux(i,j,k+1))/2.0_r8
                  END DO
               END DO
            END DO
         ELSE
            ! Phillips interpolation over two interfaces
            DO j=1,JmaxOut
               DO k=1,KmaxOut
                  DO I=1,ImaxOut
                     PresSig(i,j,k) = ((gPresaux(i,j,k)**RoCp1-gPresaux(i,j,k+1)**RoCp1)/ &
                          (RoCp1*(gPresaux(i,j,k)-gPresaux(i,j,k+1))))**RoCpR
                  END DO
               END DO
            END DO
         END IF
      END IF

      IF (GetNewTop) THEN
          TopoOut=TopoNewOut
      END If

     IF (GrADS) THEN
        WRITE (Tdef(1: 2),'(I2.2)') DateCur(1)
        WRITE (Tdef(4: 5),'(I2.2)') DateCur(3)
        WRITE (Tdef(6: 8),'(A3)')   MonChar(DateCur(2))
        WRITE (Tdef(9:12),'(I4.4)') DateCur(4)
        CALL GetGrADSInp
        IF (GrADSOnly) STOP ' GrADS Only'
     END IF

!      GridtoSpec
       OnlyTopo=.TRUE.
       CALL ICDecomposition(OnlyTopo,MendOut, ImaxOut, JmaxOut, KmaxOut,&
                            TopoOut,&
                            TopoSC)


       IF (SmoothTopo) THEN
          WRITE (UNIT=nfprt, FMT='(/,A)') ' Smoothing Topography'
          CALL SmoothCoef (TopoSC, MendOut, Kdim, MendCut)
          WRITE (UNIT=nfprt, FMT='(/,A)') ' TopoOut after Smoothing:'
          WRITE (UNIT=nfprt, FMT='(1P3G12.5)') TopoSC(1), MINVAL(TopoSC(2:)),MAXVAL(TopoSC(2:))
       END IF


!      SpectoGRid
       OnlyTopo=.TRUE.
       CALL ICRecomposition(MendOut, ImaxOut, JmaxOut, KmaxOut,&
                            TopoSC , TopoOut)


      !IF(KmaxInp==KmaxOut)
      CALL NewPs (ImaxOut, JmaxOut, KmaxInp, &
                  TopoPrs, TopoOut, PsInp, &
!                  TempOut, PresSig, PsOut, LnPsOut)
                  TempOut, PresOut, PsOut, LnPsOut)

      IF ( .not.SigmaLevel ) THEN
         DO j=1,JmaxOut
            DO k=1,KmaxOutp
               DO I=1,ImaxOut
                     !p(i,k)        = 1.e-2_r8 * a_hybr(k)  + b_hybr(k)  *  psmb(i,j)
                  gPresaux(i,j,k)    =           a_hybr(k)  + b_hybr(k)  *  PsOut(i,j)
               END DO
            END DO
            IF ( IDSLInp == 2 ) THEN
               DO k=1,KmaxOut
                  DO I=1,ImaxOut
                     PresSig(i,j,k)= 0.5_r8 * (gPresaux(i,j,k)+gPresaux(i,j,K+1))
                  END DO
               END DO
            ELSE
               ! Phillips interpolation over two interfaces
                  DO k=1,KmaxOut
                     DO I=1,ImaxOut
                        PresSig(i,j,k) = ((gPresaux(i,j,k)**RoCp1-gPresaux(i,j,k+1)**RoCp1)/ &
                             (RoCp1*(gPresaux(i,j,k)-gPresaux(i,j,k+1))))**RoCpR
                     END DO
                  END DO
            END IF
         END DO
      ELSE
         DO k=1,KmaxOut
            DO j=1,JmaxOut
               DO I=1,ImaxOut
                  PresSig(i,j,k)=PsOut(i,j)*SigLayerOut(k)
               END DO
            END DO
         END DO
      END IF


   TempOut(:,:,:)=TempOut(:,:,:)/(1.0_r8+cTv*SpHuOut(:,:,:,1))
   CALL VertSigmaInter (ImaxOut, JmaxOut, KmaxInp, KmaxOut, NTracers, &
                        PresOut, UVelOut, VVelOut, TempOut, SpHuOut, &
                        PresSig, UVelSig, VVelSig, TempSig, SpHuSig)
   TempSig(:,:,:)=TempSig(:,:,:)*(1.0_r8+cTv*SpHuSig(:,:,:,1))

     IF (GrADS) THEN
        WRITE (Tdef(1: 2),'(I2.2)') DateCur(1)
        WRITE (Tdef(4: 5),'(I2.2)') DateCur(3)
        WRITE (Tdef(6: 8),'(A3)')   MonChar(DateCur(2))
        WRITE (Tdef(9:12),'(I4.4)') DateCur(4)
        CALL GetGrADSOut
        IF (GrADSOnly) STOP ' GrADS Only'
     END IF
!
   OnlyTopo=.FALSE.
   CALL ICDecomposition(OnlyTopo,MendOut, ImaxOut, JmaxOut, KmaxOut,&
                            TopoOut,&
                            TopoSC)

!PK   CALL DectoSpherHarm (TopoOut, TopoSC, Kdim)
!PK      CALL TransSpherHarm (Kdim, TopoSC, ICaseDec)

!PK      CALL DectoSpherHarm (LnPsOut, LnPsSC, Kdim)
!PK      CALL TransSpherHarm (Kdim, LnPsSC, ICaseDec)

!PK      CALL DectoSpherHarm (TempSig, TvirSC, KmaxOut)
!PK      CALL TransSpherHarm (KmaxOut, TvirSC, ICaseDec)

!PK      CALL DectoSpherHarm (SpHuSig, SpHuSC, KmaxOut)
!PK      CALL TransSpherHarm (KmaxOut, SpHuSC, ICaseDec)

!PK      CALL UVtoDivgVort (UvelSig, VVelSig, DivgSC, VortSC)
!PK      CALL TransSpherHarm (KmaxOut, DivgSC, ICaseDec)
!PK      CALL TransSpherHarm (KmaxOut, VortSC, ICaseDec)
!



   IF(.not.SigmaLevel)THEN
   OPEN (UNIT=nfcft, FILE=TRIM(DirICnOut)//TRIM(DataOut)//'_hibrid', STATUS='REPLACE', &
                     FORM='UNFORMATTED')
                     ! a_hybr(k) * 100._r4  convert back from mb to Pa
   ELSE
   OPEN (UNIT=nfcft, FILE=TRIM(DirICnOut)//TRIM(DataOut)//'_sigma', STATUS='REPLACE', &
                     FORM='UNFORMATTED')

   END IF 
   IF ( .not.SigmaLevel ) THEN
      HybIOut=REAL(a_hybr * 100._r4, r4)
      HybLOut=REAL(b_hybr, r4)
      WRITE (UNIT=nfcft) ForecastDay, TimeOfDay, DateInp, DateCur, HybIOut, HybLOut
      WRITE (UNIT=nfprt,FMT=*) ForecastDay, TimeOfDay, DateInp, DateCur,  HybIOut, HybLOut
   ELSE
      SigIOut=REAL(SigInterOut, r4)
      SigLOut=REAL(SigLayerOut, r4)
      WRITE (UNIT=nfcft) ForecastDay, TimeOfDay, DateInp, DateCur,SigIOut,SigLOut 
      WRITE (UNIT=nfprt,FMT=*) ForecastDay, TimeOfDay, DateInp, DateCur, SigIOut,SigLOut
   END IF

   IF (GetNewTop) THEN
      WorkSC=REAL(TopoNewSC,r4)
   ELSE
      WorkSC=REAL(TopoSC,r4)
   END IF

   WRITE (UNIT=nfcft) WorkSC
   WorkSC=REAL(LnPsSC,r4)
   WRITE (UNIT=nfcft) WorkSC
   DO k=1,KmaxOut
      WorkSC=REAL(TvirSC(:,k),r4)
      WRITE (UNIT=nfcft) WorkSC
   END DO
   DO k=1,KmaxOut
      WorkSC=REAL(DivgSC(:,k),r4)
      WRITE (UNIT=nfcft) WorkSC
      WorkSC=REAL(VortSC(:,k),r4)
      WRITE (UNIT=nfcft) WorkSC
   END DO
   DO k=1,KmaxOut
      WorkSC=REAL(SpHuSC(:,k),r4)
      WRITE (UNIT=nfcft) WorkSC
   END DO
   CLOSE (UNIT=nfcft)

   WRITE (UNIT=nfprt, FMT='(/,A,I5,A,1PG13.5)') &
          ' Forecast Day = ', ForecastDay, '   Time of Day = ', TimeOfDay
   WRITE (UNIT=nfprt, FMT='(/,A,4I5)') ' Initial Date = ', DateInp
   WRITE (UNIT=nfprt, FMT='(/,A,4I5)') ' Current Date = ', DateInp
   
   IF (SigmaLevel ) THEN
      WRITE (UNIT=nfprt, FMT='(/,A)')  ' Sigma Interface:'
      WRITE (UNIT=nfprt, FMT='(7F10.6)') SigIOut
      WRITE (UNIT=nfprt, FMT='(/,A)')  ' Sigma Layer:'
      WRITE (UNIT=nfprt, FMT='(7F10.6)') SigLOut
   ELSE
      WRITE (UNIT=nfprt, FMT='(/,A)')  ' a_hybr Coef A:'
      WRITE (UNIT=nfprt, FMT='(7F10.6)') a_hybr
      WRITE (UNIT=nfprt, FMT='(/,A)')  ' b_hybr Coef B:'
      WRITE (UNIT=nfprt, FMT='(7F10.6)') b_hybr
   END IF 

   WRITE (UNIT=nfprt, FMT='(/,A)') ' Topography [m]:'
   WRITE (UNIT=nfprt, FMT='(I5,1P3G13.5)') &
          0, Cte*TopoSC(1), MINVAL(TopoSC(2:Mnwv2Out)), &
                            MAXVAL(TopoSC(2:Mnwv2Out))

   WRITE (UNIT=nfprt, FMT='(/,A)') ' Surface Pressure [hPa]:'
   WRITE (UNIT=nfprt, FMT='(I5,1P3G13.5)') &
          0, 10.0_r8*EXP(Cte*LnPsSC(1)), MINVAL(LnPsSC(2:Mnwv2Out)), &
                                         MAXVAL(LnPsSC(2:Mnwv2Out))

   WRITE (UNIT=nfprt, FMT='(/,A)') ' Virtual Temperature [K]:'
   DO k=1,KmaxOut
      WRITE (UNIT=nfprt, FMT='(I5,1P3G13.5)') &
              k, Cte*TvirSC(1,k), MINVAL(TvirSC(2:Mnwv2Out,k)), &
                                  MAXVAL(TvirSC(2:Mnwv2Out,k))
   END DO

   WRITE (UNIT=nfprt, FMT='(/,A)') ' Divergence and Vorticity [1/s]:'
   DO k=1,KmaxOut
      WRITE (UNIT=nfprt, FMT='(I5,1P3G13.5)') &
             k, Cte*DivgSC(1,k), MINVAL(DivgSC(2:Mnwv2Out,k)), &
                                 MAXVAL(DivgSC(2:Mnwv2Out,k))
      WRITE (UNIT=nfprt, FMT='(I5,1P3G13.5)') &
             k, Cte*VortSC(1,k), MINVAL(VortSC(2:Mnwv2Out,k)), &
                                 MAXVAL(VortSC(2:Mnwv2Out,k))
   END DO

   WRITE (UNIT=nfprt, FMT='(/,A)') ' Specific Humidity [g/kg]:'
   DO k=1,KmaxOut
      WRITE (UNIT=nfprt, FMT='(I5,1P3G13.5)') &
             k, 1000.0_r8*Cte*SpHuSC(1,k), &
             MINVAL(1000.0_r8*SpHuSC(2:Mnwv2Out,k)), &
             MAXVAL(1000.0_r8*SpHuSC(2:Mnwv2Out,k))
   END DO


   CALL ClsArrays ()

   CALL ClsMemFFT ()

   CALL ClsMemLegendre ()

   WRITE (UNIT=nfprt, FMT='(A)') ' '

CONTAINS

  SUBROUTINE ICDecomposition(OnlyTopo,MendOut, ImaxOut, JmaxOut, KmaxOut,&
                            TopoOut,&
                            TopoSC)

    IMPLICIT NONE
    LOGICAL, INTENT(IN) :: OnlyTopo
    INTEGER       , INTENT(IN   ) :: MendOut, ImaxOut, JmaxOut, KmaxOut
    REAL (KIND=r8), INTENT(IN   ) :: TopoOut (ImaxOut,JmaxOut)
    REAL (KIND=r8), INTENT(INOUT) :: TopoSC ((MendOut+1)*(MendOut+2))
    
!PK    CALL InitFFT
!PK    CALL InitLegendre

    IF(OnlyTopo) THEN
       WRITE (UNIT=nfprt, FMT='(/,A)') ' TopoOut before Dec:'
       WRITE (UNIT=nfprt, FMT='(1P3G12.5)') TopoSC(1), MINVAL(TopoSC(2:)), MAXVAL(TopoSC(2:))
       CALL DectoSpherHarm (TopoOut, TopoSC, Kdim)
       WRITE (UNIT=nfprt, FMT='(/,A)') ' TopoOut after Dec:'
       WRITE (UNIT=nfprt, FMT='(1P3G12.5)') TopoSC(1), MINVAL(TopoSC(2:)), MAXVAL(TopoSC(2:))
       CALL TransSpherHarm (Kdim,    TopoSC, ICaseDec)
       WRITE (UNIT=nfprt, FMT='(/,A)') ' TopoOut after Trans:'
       WRITE (UNIT=nfprt, FMT='(1P3G12.5)') TopoSC(1), MINVAL(TopoSC(2:)), MAXVAL(TopoSC(2:))
    ELSE

       WRITE (UNIT=nfprt, FMT='(/,A)') ' TopoOut before Dec:'
       WRITE (UNIT=nfprt, FMT='(1P3G12.5)') TopoSC(1), MINVAL(TopoSC(2:)), MAXVAL(TopoSC(2:))
       CALL DectoSpherHarm (TopoOut, TopoSC, Kdim)
       WRITE (UNIT=nfprt, FMT='(/,A)') ' TopoOut after Dec:'
       WRITE (UNIT=nfprt, FMT='(1P3G12.5)') TopoSC(1), MINVAL(TopoSC(2:)), MAXVAL(TopoSC(2:))
       CALL TransSpherHarm (Kdim,    TopoSC, ICaseDec)
       WRITE (UNIT=nfprt, FMT='(/,A)') ' TopoOut after Trans:'
       WRITE (UNIT=nfprt, FMT='(1P3G12.5)') TopoSC(1), MINVAL(TopoSC(2:)), MAXVAL(TopoSC(2:))

       CALL DectoSpherHarm (LnPsOut, LnPsSC, Kdim)
       CALL TransSpherHarm (Kdim,    LnPsSC, ICaseDec)

       CALL DectoSpherHarm (TempSig, TvirSC, KmaxOut)
       CALL TransSpherHarm (KmaxOut, TvirSC, ICaseDec)

       CALL DectoSpherHarm (SpHuSig, SpHuSC, KmaxOut)
       CALL TransSpherHarm (KmaxOut, SpHuSC, ICaseDec)

!PK    DO nt=1,NTracers
!PK        CALL DectoSpherHarm (SpHuSig(:,:,:,nt), SpHuSC(:,:,nt), KmaxOut)
!PK        CALL TransSpherHarm (KmaxOut, SpHuSC(:,:,nt), ICaseDec)
!PK     END DO

       CALL UVtoDivgVort (UvelSig, VVelSig, DivgSC, VortSC)
       CALL TransSpherHarm (KmaxOut, DivgSC, ICaseDec)
       CALL TransSpherHarm (KmaxOut, VortSC, ICaseDec)
    END IF

!PK    CALL ClsMemFFT
!PK    CALL ClsMemLegendre

  END SUBROUTINE ICDecomposition


  SUBROUTINE ICRecomposition(MendOut, ImaxOut, JmaxOut, KmaxOut,&
                             TopoSC , TopoOut)

    IMPLICIT NONE
    INTEGER       , INTENT(IN   ) :: MendOut, ImaxOut, JmaxOut, KmaxOut
    REAL (KIND=r8), INTENT(INOUT) :: TopoSC ((MendOut+1)*(MendOut+2))
    REAL (KIND=r8), INTENT(INOUT) :: TopoOut (ImaxOut,JmaxOut)

    CALL InitRecomposition (MendOut, ImaxOut, JmaxOut, KmaxOut)
    CALL RecompositionScalar (Kdim, TopoSC(:), TopoOut(:,:))
!PK     IF (GetNewTop .OR. SmoothTopo) THEN
!PK        WRITE (UNIT=nfprt, FMT='(/,A)') ' TopoOut before Chopping:'
!PK        WRITE (UNIT=nfprt, FMT='(1P3G12.5)') qTopoOut(1), MINVAL(qTopoOut(2:)),MAXVAL(qTopoOut(2:))
!PK        CALL Chop (qTopoOut(:), qTopoRec(:), MendOut, MendInp, Kdim)
!PK        WRITE (UNIT=nfprt, FMT='(/,A)') ' TopoRec after Chopping:'
!PK        WRITE (UNIT=nfprt, FMT='(1P3G12.5)') qTopoRec(1), MINVAL(qTopoRec(2:)),MAXVAL(qTopoRec(2:))
!PK        CALL RecompositionScalar (Kdim, qTopoRec(:), gTopoOut(:,:))
!PK        gTopoDel=gTopoOut-gTopoInp
!PK     ELSE
!PK        gTopoOut=gTopoInp
!PK     END IF
!PK 
!PK     CALL RecompositionScalar (Kdim, qLnPsInp(:), gLnPsInp(:,:))
!PK     gPsfcInp=10.0_r8*EXP(gLnPsInp)
!PK 
!PK     CALL RecompositionScalar (KmaxInp, qTvirInp, gTvirInp)
!PK 
!PK     CALL RecompositionScalar (KmaxInp, qDivgInp, gDivgInp)
!PK 
!PK     CALL RecompositionScalar (KmaxInp, qVortInp, gVortInp)
!PK 
!PK     DO nt=1,Ntracers
!PK        CALL RecompositionScalar (KmaxInp, qSpHuInp(:,:,nt), gSpHuInp(:,:,:,nt))
!PK     END DO
!PK 
!PK     CALL DivgVortToUV (qDivgInp, qVortInp, qUvelInp, qVvelInp)
!PK     CALL RecompositionVector (KmaxInp, qUvelInp, gUvelInp)
!PK     DO k=1,KmaxInp
!PK        DO j=1,JmaxOut
!PK           DO i=1,ImaxOut
!PK              gUvelInp(i,j,k)=gUvelInp(i,j,k)/coslat(j)
!PK           END DO
!PK        END DO
!PK     END DO
!PK     CALL RecompositionVector (KmaxInp, qVvelInp, gVvelInp)
!PK     DO k=1,KmaxInp
!PK        DO j=1,JmaxOut
!PK           DO i=1,ImaxOut
!PK              gVvelInp(i,j,k)=gVvelInp(i,j,k)/coslat(j)
!PK           END DO
!PK        END DO
!PK    END DO

    CALL ClsMemRecomposition
  END SUBROUTINE ICRecomposition


SUBROUTINE SmoothCoef (qOut, MendO, Kmax, MendC)

! Smoothes Spherical Harmonics Coefficients
!          Using Hoskin's(?) Filter

  IMPLICIT NONE

  INTEGER, INTENT (IN) :: MendO, Kmax, MendC
!  Mend1Out=MendOut+1
!  Mend2Out=MendOut+2
!  Mnwv2Out=Mend1Out*(MendOut+2)
!  TopoNewSC (Mnwv2Out)
  REAL (KIND=r8), DIMENSION ((MendO+1)*(MendO+2),Kmax), INTENT (IN OUT) :: qOut

  INTEGER :: k, mn, nn, mm, Mmax, mns

  REAL (KIND=r8) :: rm, rn, cx, red, rmn, ck

  REAL (KIND=r8), DIMENSION ((MendO+1)*(MendO+2)) :: Weight

  INTEGER :: icall=0
  SAVE :: icall

  WRITE (UNIT=nfprt, FMT='(/,A,I5)') ' Doing Smoothing : ', icall

  IF (icall == 0) THEN
    rm=REAL(2*MendC,r8)*REAL(2*MendC+1,r8)
    rn=REAL(MendC-1,r8)*REAL(MendC,r8)
    red=(SmthPerCut)**(-(rm*rm)/(rn*rn)/Iter)
    WRITE (UNIT=nfprt, FMT='(/,A,F10.4)') ' From SmoothCoef: Red = ', red
    cx=-LOG(red)/(rm*rm)
    mn=0
    DO nn=1,MendO+1
      Mmax=MendO-nn+2
      DO mm=1,Mmax
        mn=mn+1
        mns=mm+nn-1
        rmn=REAL(mns-1,r8)*REAL(mns,r8)
        ck=(EXP(cx*rmn*rmn))**Iter
        Weight(2*mn-1)=ck
        Weight(2*mn)=ck
      END DO
    END DO
    icall=icall+1
  END IF

  DO k=1,Kmax
    DO mn=1,(MendO+1)*(MendO+2)
      qOut(mn,k)=qOut(mn,k)*Weight(mn)
    END DO
  END DO

END SUBROUTINE SmoothCoef


  SUBROUTINE GetGrADSInp

    IMPLICIT NONE

    WRITE (UNIT=nfprt, FMT='(/,A,L6,/)') ' GrADS     = ', GrADS
    WRITE (UNIT=nfprt, FMT='(/,A,L6,/)') ' GrADSOnly = ', GrADSOnly

    INQUIRE (IOLENGTH=IOL) gWorkOut
    OPEN (UNIT=nfgrd, FILE=TRIM(DirICnOut)//TRIM(DataOut)//'.GrADS', FORM='UNFORMATTED', &
         ACCESS='DIRECT', RECL=IOL, ACTION='WRITE', STATUS='REPLACE', IOSTAT=ios)
    IF (ios /= 0) THEN
       WRITE (UNIT=nferr, FMT='(3A,I4)') ' ** (Error) ** Open file ', &
            TRIM(TRIM(DirICnOut)//TRIM(DataOut))//'.GrADS', &
            ' returned IOStat = ', ios
       STOP ' ** (Error) **'
    END IF
    gWorkOut=REAL(TopoOut(:,:),kind=r4)
    nRec=1
    WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    gWorkOut=REAL(PsInp(:,:),kind=r4)
    nRec=2
    WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    DO k=1,KmaxInp
       gWorkOut=REAL(TempOut(:,:,k),kind=r4)
       nRec=2+INT(k)
       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    END DO
!    DO k=1,KmaxInp
!       gWorkOut=gDivgInp(:,:,k)
!       nRec=2+INT(k+KmaxInp)
!       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
!    END DO
!    DO k=1,KmaxInp
!       gWorkOut=gVortInp(:,:,k)
!       nRec=2+INT(k+2*KmaxInp)
!       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
!    END DO
    DO k=1,KmaxInp
       gWorkOut=REAL(SpHuOut(:,:,k,1),kind=r4)
       nRec=2+INT(k+1*KmaxInp)
       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    END DO
    DO k=1,KmaxInp
       gWorkOut=REAL(UVelOut(:,:,k),kind=r4)
       nRec=2+INT(k+2*KmaxInp)
       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    END DO
    DO k=1,KmaxInp
       gWorkOut=REAL(VVelOut(:,:,k),kind=r4)
       nRec=2+INT(k+3*KmaxInp)
       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    END DO
!    IF (GetOzone) THEN
!       nt=2
!       DO k=1,KmaxInp
!          gWorkOut=gSpHuInp(:,:,k,nt)
!          nRec=2+INT(k+6*KmaxInp)
!          WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
!       END DO
!    END IF
!    IF (GetTracers) THEN
!       DO nt=3,NTracers
!          DO k=1,KmaxInp
!             gWorkOut=gSpHuInp(:,:,k,nt)
!             nRec=2+INT(k+(4+nt)*KmaxInp)
!             WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
!          END DO
!       END DO
!    END IF
    CLOSE (UNIT=nfgrd)

    OPEN (UNIT=nfctl, FILE=TRIM(DirICnOut)//TRIM(DataOut)//'.GrADS.ctl', FORM='FORMATTED', &
         ACCESS='SEQUENTIAL', ACTION='WRITE', STATUS='REPLACE', IOSTAT=ios)
    IF (ios /= 0) THEN
       WRITE (UNIT=nferr, FMT='(3A,I4)') ' ** (Error) ** Open file ', &
            TRIM(TRIM(DirICnOut)//TRIM(DataOut))//'.GrADS.ctl', &
            ' returned IOStat = ', ios
       STOP ' ** (Error) **'
    END IF
    WRITE (UNIT=nfctl, FMT='(A)') 'dset ^'//TRIM(DataOut)//'.GrADS'
    WRITE (UNIT=nfctl, FMT='(A)') 'options yrev big_endian'
    WRITE (UNIT=nfctl, FMT='(A)') 'undef 1e20'
    WRITE (UNIT=nfctl, FMT='(A,I5,A,2F12.6)') 'xdef ',ImaxOut,' linear ', &
         0.0_r8, 360.0_r8/REAL(ImaxOut,r8)
    WRITE (UNIT=nfctl, FMT='(A,I5,A)') 'ydef ',JmaxOut,' levels '
    WRITE (UNIT=nfctl, FMT='(6F10.3)') glat(JmaxOut:1:-1)
    WRITE (UNIT=nfctl, FMT='(A,I5,A)') 'zdef ',KmaxInp,' levels '
    WRITE (UNIT=nfctl, FMT='(6F10.3)') PresInp
    WRITE (UNIT=nfctl, FMT='(3A)') 'tdef 1 linear ', Tdef,' 1dy'
    IF (NTracers == 1) THEN
       WRITE (UNIT=nfctl, FMT='(A)') 'vars 6'
    ELSE
       WRITE (UNIT=nfctl, FMT='(A,I5)') 'vars ',6+NTracers
    END IF
    WRITE (UNIT=nfctl, FMT='(A)') 'topo   0 99 Topography        '//TruncOut//' (m)'
    WRITE (UNIT=nfctl, FMT='(A)') 'pslc   0 99 Surface Pressure  '//TruncOut//' (hPa)'
    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'tvir ',KmaxInp,' 99 Virt Temperature  '// &
         TruncOut//' (K)'
!    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'divg ',KmaxInp,' 99 Divergence        '// &
!         TruncOut//' (1/s)'
!    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'vort ',KmaxInp,' 99 Vorticity         '// &
!         TruncOut//' (1/s)'
    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'umes ',KmaxInp,' 99 Specific Humidity '// &
         TruncOut//' (kg/kg)'
    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'uvel ',KmaxInp,' 99 Zonal Wind        '// &
         TruncOut//' (m/s)'
    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'vvel ',KmaxInp,' 99 Meridional Wind   '// &
         TruncOut//' (m/s)'
!    IF (GetOzone) THEN
!       WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'ozon ',KmaxInp,' 99 Ozone             '// &
!            TruncOut//' (?)'
!    END IF
!    IF (GetTracers) THEN
!       DO nt=3,NTracers
!          WRITE (UNIT=nfctl, FMT='(A,I1,A,I3,A)') 'trc',nt-2,' ',KmaxInp, &
!               ' 99 Tracer            '//TruncInp//' (?)'
!       END DO
!    END IF
    WRITE (UNIT=nfctl, FMT='(A)') 'endvars'
    CLOSE (UNIT=nfctl)

  END SUBROUTINE GetGrADSInp

  SUBROUTINE GetGrADSOut

    IMPLICIT NONE

    WRITE (UNIT=nfprt, FMT='(/,A,L6,/)') ' GrADS     = ', GrADS
    WRITE (UNIT=nfprt, FMT='(/,A,L6,/)') ' GrADSOnly = ', GrADSOnly

    INQUIRE (IOLENGTH=IOL) gWorkOut
    OPEN (UNIT=nfgrd, FILE=TRIM(DirICnOut)//TRIM(DataOut)//'.GrADSOut', FORM='UNFORMATTED', &
         ACCESS='DIRECT', RECL=IOL, ACTION='WRITE', STATUS='REPLACE', IOSTAT=ios)
    IF (ios /= 0) THEN
       WRITE (UNIT=nferr, FMT='(3A,I4)') ' ** (Error) ** Open file ', &
            TRIM(TRIM(DirICnOut)//TRIM(DataOut))//'.GrADSOut', &
            ' returned IOStat = ', ios
       STOP ' ** (Error) **'
    END IF
    gWorkOut=REAL(TopoOut(:,:),kind=r4)
    nRec=1
    WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    gWorkOut=REAL(PsOut(:,:),kind=r4)
    nRec=2
    WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    DO k=1,KmaxOut
       gWorkOut=REAL(TempSig(:,:,k),kind=r4)
       nRec=2+INT(k)
       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    END DO
!    DO k=1,KmaxOut
!       gWorkOut=gDivgInp(:,:,k)
!       nRec=2+INT(k+KmaxOut)
!       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
!    END DO
!    DO k=1,KmaxOut
!       gWorkOut=gVortInp(:,:,k)
!       nRec=2+INT(k+2*KmaxOut)
!       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
!    END DO
    DO k=1,KmaxOut
       gWorkOut=REAL(SpHuSig(:,:,k,1),kind=r4)
       nRec=2+INT(k+1*KmaxOut)
       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    END DO
    DO k=1,KmaxOut
       gWorkOut=REAL(UVelSig(:,:,k),kind=r4)
       nRec=2+INT(k+2*KmaxOut)
       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    END DO
    DO k=1,KmaxOut
       gWorkOut=REAL(VVelSig(:,:,k),kind=r4)
       nRec=2+INT(k+3*KmaxOut)
       WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
    END DO
!    IF (GetOzone) THEN
!       nt=2
!       DO k=1,KmaxOut
!          gWorkOut=gSpHuInp(:,:,k,nt)
!          nRec=2+INT(k+6*KmaxOut)
!          WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
!       END DO
!    END IF
!    IF (GetTracers) THEN
!       DO nt=3,NTracers
!          DO k=1,KmaxOut
!             gWorkOut=gSpHuInp(:,:,k,nt)
!             nRec=2+INT(k+(4+nt)*KmaxOut)
!             WRITE (UNIT=nfgrd, REC=nRec) gWorkOut
!          END DO
!       END DO
!    END IF
    CLOSE (UNIT=nfgrd)

    OPEN (UNIT=nfctl, FILE=TRIM(DirICnOut)//TRIM(DataOut)//'.GrADSOut.ctl', FORM='FORMATTED', &
         ACCESS='SEQUENTIAL', ACTION='WRITE', STATUS='REPLACE', IOSTAT=ios)
    IF (ios /= 0) THEN
       WRITE (UNIT=nferr, FMT='(3A,I4)') ' ** (Error) ** Open file ', &
            TRIM(TRIM(DirICnOut)//TRIM(DataOut))//'.GrADS.ctl', &
            ' returned IOStat = ', ios
       STOP ' ** (Error) **'
    END IF
    WRITE (UNIT=nfctl, FMT='(A)') 'dset ^'//TRIM(DataOut)//'.GrADSOut'
    WRITE (UNIT=nfctl, FMT='(A)') 'options yrev big_endian'
    WRITE (UNIT=nfctl, FMT='(A)') 'undef 1e20'
    WRITE (UNIT=nfctl, FMT='(A,I5,A,2F12.6)') 'xdef ',ImaxOut,' linear ', &
         0.0_r8, 360.0_r8/REAL(ImaxOut,r8)
    WRITE (UNIT=nfctl, FMT='(A,I5,A)') 'ydef ',JmaxOut,' levels '
    WRITE (UNIT=nfctl, FMT='(6F10.3)') glat(JmaxOut:1:-1)
    WRITE (UNIT=nfctl, FMT='(A,I5,A)') 'zdef ',KmaxOut,' levels '
    IF (SigmaLevel ) THEN
       WRITE (UNIT=nfctl, FMT='(6F10.3)') 1000*SigLayerOut
    ELSE
       WRITE (UNIT=nfctl, FMT='(6F10.3)') a_hybr(1:KmaxOut)   + b_hybr(1:KmaxOut) * 1000.0
    END IF
    WRITE (UNIT=nfctl, FMT='(3A)') 'tdef 1 linear ', Tdef,' 1dy'
    IF (NTracers == 1) THEN
       WRITE (UNIT=nfctl, FMT='(A)') 'vars 6'
    ELSE
       WRITE (UNIT=nfctl, FMT='(A,I5)') 'vars ',6+NTracers
    END IF
    WRITE (UNIT=nfctl, FMT='(A)') 'topo   0 99 Topography        '//TruncOut//' (m)'
    WRITE (UNIT=nfctl, FMT='(A)') 'pslc   0 99 Surface Pressure  '//TruncOut//' (hPa)'
    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'tvir ',KmaxOut,' 99 Virt Temperature  '// &
         TruncOut//' (K)'
!    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'divg ',KmaxOut,' 99 Divergence        '// &
!         TruncOut//' (1/s)'
!    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'vort ',KmaxOut,' 99 Vorticity         '// &
!         TruncOut//' (1/s)'
    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'umes ',KmaxOut,' 99 Specific Humidity '// &
         TruncOut//' (kg/kg)'
    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'uvel ',KmaxOut,' 99 Zonal Wind        '// &
         TruncOut//' (m/s)'
    WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'vvel ',KmaxOut,' 99 Meridional Wind   '// &
         TruncOut//' (m/s)'
!    IF (GetOzone) THEN
!       WRITE (UNIT=nfctl, FMT='(A,I3,A)') 'ozon ',KmaxOut,' 99 Ozone             '// &
!            TruncOut//' (?)'
!    END IF
!    IF (GetTracers) THEN
!       DO nt=3,NTracers
!          WRITE (UNIT=nfctl, FMT='(A,I1,A,I3,A)') 'trc',nt-2,' ',KmaxOut, &
!               ' 99 Tracer            '//TruncInp//' (?)'
!       END DO
!    END IF
    WRITE (UNIT=nfctl, FMT='(A)') 'endvars'
    CLOSE (UNIT=nfctl)

  END SUBROUTINE GetGrADSOut
END PROGRAM SpectralIntialConditions
